/* compare two versions
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/*
 * libdpkg - Debian packaging suite library routines
 * vercmp.c - comparison of version numbers
 *
 * Copyright (C) 1995 Ian Jackson <iwj10@cus.cam.ac.uk>
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with dpkg; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <ctype.h>

static int verrevcmp(const char *val, const char *ref) {
  int vc, rc;
  long vl, rl;
  const char *vp, *rp;

  if (!val) val= "";
  if (!ref) ref= "";
  for (;;) {
    vp= val;  while (*vp && !isdigit(*vp)) vp++;
    rp= ref;  while (*rp && !isdigit(*rp)) rp++;
    for (;;) {
      vc= val == vp ? 0 : *val++;
      rc= ref == rp ? 0 : *ref++;
      if (!rc && !vc) break;
      if (vc && !isalpha(vc)) vc += 256; /* assumes ASCII character set */
      if (rc && !isalpha(rc)) rc += 256;
      if (vc != rc) return vc - rc;
    }
    val= vp;
    ref= rp;
    vl=0;  if (isdigit(*vp)) vl= strtol(val,(char**)&val,10);
    rl=0;  if (isdigit(*rp)) rl= strtol(ref,(char**)&ref,10);
    if (vl != rl) return vl - rl;
    if (!*val && !*ref) return 0;
    if (!*val) return -1;
    if (!*ref) return +1;
  }
}

int versioncompare(const char *version, const char *revision,
                   const char *refversion, const char *refrevision) {
  int r;
  r= verrevcmp(version,refversion);  if (r) return r;
  return verrevcmp(revision,refrevision);
}

void usage_and_die(void)
{
  fprintf(stderr, 
	  "dvercmp <ver> <rev> <nver> <nrev>\n"
	  "  ver,rev = current version/revision\n"
	  "  nver,nrev = new version/revision\n"
	  "  all args can be 'none' for the empty string\n"
	  "  exit status is 0 if new > current\n"
	  "  else 1\n");
  exit(1);
}

int 
main(int argc, char **argv)
{
  char *none = "";
  char *strs[4];
  int i;

  if(argc != 5) usage_and_die();

  for(i=0; i<4; i++) {
    if(!strcmp(argv[i+1], "none")) strs[i] = none;
    else strs[i] = argv[i+1];
  }
  i = versioncompare(strs[0], strs[1], strs[2], strs[3]);
  if(i==0) printf("same version\n");
  if(i>0) printf("older\n");
  if(i<0) {
    printf("newer\n");
    exit(0);
  }
  exit(1);
}
